package name.nathanmcrae.numbersstation;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.Level;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;
import org.apache.commons.cli.*;

public class Main extends Application {
    private static final Logger logger = Logger.getLogger(Main.class.getName());
    // TODO: get git info
    private static final String VERSION = "0.0.1";
    private static Path statePath = null;

    public Path getStatePath() {
        if (statePath == null) {
            String stateHome = System.getenv("XDG_STATE_HOME");
            if (stateHome == null || stateHome.isEmpty() || !Paths.get(stateHome).isAbsolute()) {
                String userHome = System.getProperty("user.home");
                statePath = Paths.get(userHome, ".local", "state", "numbers-station");
            } else {
                statePath = Paths.get(stateHome, "numbers-station");
            }
        }

        return statePath;
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        setupLogger();
        Parent root = FXMLLoader.load(getClass().getResource("MainView.fxml"));
        primaryStage.setTitle("Numbers Station");
        primaryStage.setScene(new Scene(root));
        primaryStage.show();
        logger.info("Application started");
    }

    private void setupLogger() {
        try {
            Path logFile = getStatePath().resolve("main.log");

            if (!Files.exists(logFile.getParent())) {
                Files.createDirectories(logFile.getParent());
            }

            FileHandler fileHandler = new FileHandler(logFile.toString(), true);
            fileHandler.setFormatter(new CustomFormatter());
            logger.addHandler(fileHandler);
        } catch (IOException e) {
            System.out.println("Failed to set up logger: " + e.getMessage());
        }
    }

    private static void parseArguments(String[] args) {
        Options options = new Options();

        Option help = new Option("h", "help", false, "Show help");
        options.addOption(help);

        Option version = new Option("v", "version", false, "Show version");
        options.addOption(version);

        Option station = new Option("s", "station", true, "Specify station name");
        station.setArgName("station-name");
        options.addOption(station);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        String stationName = null;

        try {
            CommandLine cmd = parser.parse(options, args);

            if (cmd.hasOption("help")) {
                formatter.printHelp("numbers-station", options);
                System.exit(0);
            }

            if (cmd.hasOption("version")) {
                System.out.println("Numbers Station version " + VERSION);
                System.exit(0);
            }

            if (cmd.hasOption("station")) {
                stationName = cmd.getOptionValue("station");
            }
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp("numbers-station", options);
            System.exit(1);
        }
    }

    public static void main(String[] args) {
        parseArguments(args);

        Thread.setDefaultUncaughtExceptionHandler((thread, throwable) -> {
            logger.log(Level.SEVERE, "Unhandled exception caught", throwable);
        });

        Platform.setImplicitExit(false);
        Thread.currentThread().setUncaughtExceptionHandler((thread, throwable) -> {
            logger.log(Level.SEVERE, "Unhandled exception in JavaFX application thread", throwable);
        });

        launch(args);
    }
}
